/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_FLDR_HPP_
#define _HEAVY_CONTEXT_WSTD_FLDR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvMath.h"
#include "HvSignalVar.h"
#include "HvSignalEnvelope.h"
#include "HvControlCast.h"
#include "HvControlIf.h"
#include "HvSignalLine.h"
#include "HvControlVar.h"
#include "HvControlBinop.h"

class Heavy_WSTD_FLDR : public HeavyContext {

 public:
  Heavy_WSTD_FLDR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_FLDR();

  const char *getName() override { return "WSTD_FLDR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        FLDR = 0xE42C892E, // Fldr
        LMTR = 0xCB23DC60, // Lmtr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_QSPuZbv8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_TzOnom78_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_dF708nWk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_HS2tDz3A_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ipo4jrG2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_uZ0aDb99_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_ZKPu3QVt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_l3uHNbsq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_CWwNvEY9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_DNkOGIaE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_LxqY7MSx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_h7A6IYej_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_dWCO80FE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_gmxCFBvB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_R5QBCk6x_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_bP3YcNK8_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_2221B4mV_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_ZsPkxTNI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_FXP0IIkS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MlMGSVO5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_V7QET7Xg_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_v5n18GEY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_DTLhE8Bv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_ES5HR1YK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_aHFzvM4P_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_8rJEzMDI_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_SJZgVO2s_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4Ayw8xAl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_XRgDmgF2_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_srJmS8tc_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_Ec2nXk9M_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_szeiBHHr_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_wgVKUf0J_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cIf_RxcjtERl_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_6IMuB0G6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void sEnv_m99gwdAo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vDiNcATE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_dWmh7zbE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_vQlLoJ6w_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_y4ASm0Zi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_a7YUlOn9_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_qXzx82LB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Q1SnR8lZ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_UMQAUAsS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_x7MPpDJ5_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_YnEupfGi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_I9N7l5qL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_2D5hFIAH_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_eR7pr7xm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1HvsZmie_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_lu0hdqvY_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_1MGmorxB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_WhMjC6IB_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_2jyOdiDi_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_G7DbwElE_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_j8eNFfGd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_iUsBOwjw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_3WmoOb8V_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_YLt30uQs_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_ffi8icBQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_C8XNLGNK_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_rGbj4c7C_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_attqrRFZ;
  SignalEnvelope sEnv_ZsPkxTNI;
  SignalEnvelope sEnv_FXP0IIkS;
  SignalEnvelope sEnv_6IMuB0G6;
  SignalEnvelope sEnv_m99gwdAo;
  SignalLine sLine_kLU7eO0E;
  SignalLine sLine_4gDzT7tR;
  ControlBinop cBinop_QSPuZbv8;
  ControlBinop cBinop_TzOnom78;
  ControlBinop cBinop_dF708nWk;
  ControlBinop cBinop_HS2tDz3A;
  ControlBinop cBinop_ZKPu3QVt;
  ControlBinop cBinop_CWwNvEY9;
  ControlBinop cBinop_DNkOGIaE;
  ControlBinop cBinop_LxqY7MSx;
  ControlVar cVar_h7A6IYej;
  ControlVar cVar_gmxCFBvB;
  SignalVarf sVarf_aIi3QzhP;
  ControlVar cVar_R5QBCk6x;
  ControlIf cIf_bP3YcNK8;
  ControlIf cIf_2221B4mV;
  ControlBinop cBinop_MlMGSVO5;
  ControlBinop cBinop_v5n18GEY;
  ControlBinop cBinop_DTLhE8Bv;
  ControlBinop cBinop_8rJEzMDI;
  ControlBinop cBinop_4Ayw8xAl;
  SignalVarf sVarf_HgMYSNNg;
  ControlVar cVar_szeiBHHr;
  ControlIf cIf_wgVKUf0J;
  ControlIf cIf_RxcjtERl;
  ControlBinop cBinop_vDiNcATE;
  ControlBinop cBinop_vQlLoJ6w;
  ControlBinop cBinop_y4ASm0Zi;
  ControlBinop cBinop_Q1SnR8lZ;
  ControlBinop cBinop_x7MPpDJ5;
  ControlVar cVar_eR7pr7xm;
  ControlBinop cBinop_1HvsZmie;
  SignalVarf sVarf_995xMt5u;
  SignalVarf sVarf_8oDPmAxu;
  SignalVarf sVarf_A1DbSCOJ;
  SignalVarf sVarf_3e0cdaw1;
  ControlBinop cBinop_2jyOdiDi;
};

#endif // _HEAVY_CONTEXT_WSTD_FLDR_HPP_
